using Serilog;

namespace TwitchChatTTS.Hermes.Socket.Requests
{
    public class DeleteTTSFilterAck : IRequestAck
    {
        public string Name => "delete_tts_filter";
        private readonly User _user;
        private readonly ILogger _logger;

        public DeleteTTSFilterAck(User user, ILogger logger)
        {
            _user = user;
            _logger = logger;
        }

        public void Acknowledge(string requestId, string json, IDictionary<string, object>? requestData)
        {
            if (requestData == null)
            {
                _logger.Warning("Request data is null.");
                return;
            }

            var id = requestData["id"].ToString();
            if (string.IsNullOrEmpty(id))
            {
                _logger.Warning($"Filter Id is invalid [filter id: {id}]");
                return;
            }
            var filter = _user.RegexFilters.FirstOrDefault(f => f.Id == id);
            if (filter == null)
            {
                _logger.Warning($"Filter Id does not exist [filter id: {id}]");
                return;
            }

            _user.RegexFilters.Remove(filter);
            _logger.Information($"Deleted a filter [filter id: {id}][search: {filter.Search}][replace: {filter.Replace}]");
        }
    }
}